%--------------------------------------------------------------------------
% This is a demo for the BBQ method in [1], using an unconstrained quadratic
% example in [2]
% 
%           min  0.5*(x-xs)'*A*(x-xs)
% 
% Stop condition: max(abs(g))<=tol
% 
% Reference: 
%   [1] Yakui Huang, Yu-Hong Dai, Xin-Wei Liu, Equipping Barzilai-Borwein method 
%       with two dimensional quadratic termination property, 2021. 
%   [2] Y.-X. Yuan, A new stepsize for the steepest descent method, 
%       J. Comput. Math., 24 (2006), pp. 149-156.
%
% Authors:
%       Yakui Huang, Yu-Hong Dai, Xin-Wei Liu
%       contact hyk@hebut.edu.cn; dyh@lsec.cc.ac.cn; math-lxw@hebut.edu.cn
% 
% Last Update: Jan 8, 2021
%--------------------------------------------------------------------------

clear

n = 10000;
maxiter = 20000;
tol = 1e-6;
kappa = 1e4;
x0 = zeros(n,1);

d = 1 + (kappa-1).*rand(n,1);
d(1) = 1; d(end) = kappa;
A = d(:);
xs = -10 + 20*rand(n,1);

if size(A,2) == 1
    grad = @(x) A.*(x - xs);
    fobj = @(x) (x - xs)'*(A.*(x - xs));
    fAx = @(x) A.*x;
else
    grad = @(x) A*(x - xs);
    fobj = @(x) (x - xs)'*(A*(x - xs));
    fAx = @(x) A*x;
end

opts.PrintLevel = 1;
[out, hist] = BBQ_unc(x0, fobj, grad, opts);
